"""
Python Markdown

A Python implementation of John Gruber's Markdown.

Documentation: https://python-markdown.github.io/
GitHub: https://github.com/Python-Markdown/markdown/
PyPI: https://pypi.org/project/Markdown/

Started by Manfred Stienstra (http://www.dwerg.net/).
Maintained for a few years by Yuri Takhteyev (http://www.freewisdom.org).
Currently maintained by Waylan Limberg (https://github.com/waylan),
Dmitry Shachnev (https://github.com/mitya57) and Isaac Muse (https://github.com/facelessuser).

Copyright 2007-2023 The Python Markdown Project (v. 1.7 and later)
Copyright 2004, 2005, 2006 Yuri Takhteyev (v. 0.2-1.6b)
Copyright 2004 Manfred Stienstra (the original version)

License: BSD (see LICENSE.md for details).
"""

from markdown.test_tools import TestCase


class TestRawHtml(TestCase):
    def test_inline_html_angle_brackets(self):
        self.assertMarkdownRenders("<span>e<c</span>", "<p><span>e&lt;c</span></p>")
        self.assertMarkdownRenders("<span>e>c</span>", "<p><span>e&gt;c</span></p>")
        self.assertMarkdownRenders("<span>e < c</span>", "<p><span>e &lt; c</span></p>")
        self.assertMarkdownRenders("<span>e > c</span>", "<p><span>e &gt; c</span></p>")

    def test_inline_html_backslashes(self):
        self.assertMarkdownRenders('<img src="..\\..\\foo.png">', '<p><img src="..\\..\\foo.png"></p>')

    def test_noname_tag(self):
        self.assertMarkdownRenders('<span></></span>', '<p><span>&lt;/&gt;</span></p>')

    def test_markdown_nested_in_inline_comment(self):
        self.assertMarkdownRenders(
            'Example: <!-- [**Bold link**](http://example.com) -->',
            '<p>Example: <!-- <a href="http://example.com"><strong>Bold link</strong></a> --></p>'
        )
