{% extends "_base/attribute.html.jinja" %}

{% block heading scoped %}
  <a class="doc-source-link" href="{{ config.extra.source.repo }}/tree/{{ config.extra.source.tag }}/{{ attribute.relative_filepath }}#L{{ attribute.lineno }}{% if attribute.endlineno > attribute.lineno %}-L{{ attribute.endlineno }}{% endif %}" title='View source code on GitHub'>&lsaquo;&rsaquo;</a>
  {% if config.show_symbol_type_heading %}<code class="doc-symbol doc-symbol-heading doc-symbol-attribute"></code>{% endif %}
  {%+ filter highlight(language="python", inline=True) %}
    {{ attribute_name }}{% if attribute.annotation %}: {{ attribute.annotation }}{% endif %}
  {% endfilter %}
{% endblock heading %}

{% block contents scoped %}
  {{ super() }}
  {% if attribute.value %}
  <p>Defined Value:</p>
  {%+ filter highlight(language="python", inline=False) %}
{{ attribute.source }}
  {% endfilter %}
  {% endif %}
{% endblock contents %}
