{% extends "_base/class.html.jinja" %}

{% block heading scoped %}
  <a class="doc-source-link" href="{{ config.extra.source.repo }}/tree/{{ config.extra.source.tag }}/{{ class.relative_filepath }}#L{{ class.lineno }}" title="{{ config.extra.source.title }}">&lsaquo;&rsaquo;</a>
  {{ super() }}
{% endblock heading %}
